---------------------------------------------------
-- Release 1 --  Created by Lucius L. Hilley III --
---------------------------------------------------
-- E-Mail luciuslhilleyiii@juno.com --
-- OR Euphoria list                 --
--------------------------------------

include sort.e
--sort.e is used to sort Color_Used() so that Color_Unused()
--can work fast.
--  NOTE: Color_Unused() will not work without sort().

--ALL references to image require the {palette, image}
--combonation that is Return when using read_bitmap()
--and PCX_Load().

--Color_Used(image)
--  finds all the Color palette values Used in image.
--Color_Unused(image)
--  finds all the Color palette values Unused in image.
--Color_Swap(image, color1, color2)
--  Swaps color1 with color2
--  color1 & color2 are Color palette values.
--  All references to these values are swapped in the image.
--  The {R, G, B} values for color1 & color2 are swapped in
--  the palette.  The will palette & image are changed.
--  Both used togother will appear identical to the original
--  palette & image combonation.
--Common({sequence1, sequence2, ...})
--  returns the values that are common to all sequences the
--  that are passed to it.
--Mix_Palette(image1, image2)
--  Trys to return both images with a new palette that they
--  both can agree on.
--  success returns {0, image1, image2} New images.
--  faliure returns {-1}

global function Filter_Image(sequence fimage, integer color)
  fimage = (fimage = color)
  fimage = fimage * 255
  return fimage
end function

global function Mix_Image(sequence fore, sequence back, sequence filter)
  filter = and_bits(filter, back)
  filter = or_bits(filter, fore)
  return filter
end function

global function Color_Used(sequence image)
  integer up
  sequence used
  
  used = repeat(256, 256)
  up = 0
  for A = 1 to length(image[2]) do
    for B = 1 to length(image[2][A]) do
      if find(image[2][A][B], used[1..up]) = 0 then
	up = up + 1
	used[up] = image[2][A][B]
      end if
    end for
  end for
  
  return used[1..up]
end function

global function Color_Unused(sequence image)
  integer l, p
  sequence used, unused
  
  used = Color_Used(image)
  
  if length(used) = 256 then
    return {-1}
  end if
  
  used = sort(used)
  
  l = length(used)
  used = -1 & used & 256
  unused = repeat(0, 256 - l)
  p = 1
  for A = 1 to l + 1 do
    for B = used[A] + 1 to used[A + 1] - 1 do
      unused[p] = B
      p = p + 1
    end for
  end for
  
  return unused
end function

global function Color_Swap(sequence image, integer color1, integer color2)
  integer chk, p
  sequence dummy

  dummy = image[1][color1 + 1]
  image[1][color1 + 1] = image[1][color2 + 1]
  image[1][color2 + 1] = dummy

  for A = 1 to length(image[2]) do
    chk = find(color2, image[2][A])
    p = chk
    while chk do
      image[2][A][p] = 256
      chk = find(color2, image[2][A][p + 1..length(image[2][A])])
      p = p + chk
    end while
  end for
  for A = 1 to length(image[2]) do
    chk = find(color1, image[2][A])
    p = chk
    while chk do
      image[2][A][p] = color2
      chk = find(color1, image[2][A][p + 1..length(image[2][A])])
      p = p + chk
    end while
  end for
  for A = 1 to length(image[2]) do
    chk = find(256, image[2][A])
    p = chk
    while chk do
      image[2][A][p] = color1
      chk = find(256, image[2][A][p + 1..length(image[2][A])])
      p = p + chk
    end while
  end for
  return image
end function

global function Color_Swap2(sequence image, integer color1, integer color2)
  sequence t
  sequence t1, t2, t3, t4
  sequence t5, t6, t7, t8
  sequence t9, tA, tB

  if color1 != color2 then
    t = image[1][color1 + 1]
    image[1][color1 + 1] = image[1][color2 + 1]
    image[1][color2 + 1] = t

    t = repeat(repeat(0, length(image[2][1])), length(image[2]))
    t1 = (image[2] = color1)
    if compare(t, t1) then
      t2 = t1 * (color1 + 1)
      t3 = image[2] + 1
      t4 = xor_bits(t2, t3)
    else
      t4 = image[2] + 1
    end if

    t5 = (image[2] = color2)
    if compare(t, t5) then
	t6 = t5 * (color2 + 2)
	t7 = t4 + 1
	t8 = xor_bits(t6, t7)
    else
	t8 = t4 + 1
    end if
  
    if compare(t, t5) then
      t9 = t5 * (color1 + 2)
      tA = or_bits(t8, t9) - 1
    else
      tA = t8 - 1
    end if

    if compare(t, t1) then
      tB = t1 * (color2 + 1)
      image[2] = or_bits(tA, tB) - 1
    else
      image[2] = tA - 1
    end if
  end if
  return image
end function

global function Common(sequence seqs)
  integer f
  sequence all, common
  
  all = seqs[1]
  for A = 2 to length(seqs) do
    all = all & seqs[A]
  end for

  common = {}
  for A = 1 to length(all) do
    if find(all[A], common) = 0 then
      f = 1
      for B = 1 to length(seqs) do
	if find(all[A], seqs[B]) = 0 then
	  f = 0
	  exit
	end if
      end for
    end if
    if f then
      common = common & all[A]
    end if
  end for

  return common
end function

--function Mix_Palette(sequence image1, sequence image2)
  --image1 = image2

  --return {-1}
--end function

